<?php

if(defined('OP5_CGI_AUTH_LOADER')) die('No circular loading here...');
define('OP5_CGI_AUTH_LOADER', 1);

require_once( 'op5/auth/Auth.php' );

/* ***************************************
 * Setup Configuration
 * ***************************************/

$webroot_basepath      = '/opt/monitor/share';

$cgi_basepath          = '/opt/monitor/sbin';
$cgi_error_log         = '/dev/null';
$cgi_cfg_path          = tempnam('/tmp','cgi_cfg.');
$cgi_cfg_unlink        = true; /* remove file after complete run. set true when using tempnam */
$cgi_cfg_template_path = '/opt/monitor/etc/cgi.cfg';
$cgi_bin_urlbase       = '/cgi-bin';

$url_html_path         = $_SERVER['SCRIPT_NAME'];

$document_root         = '/var/www/html';
$allowed_dirs          = array(
	$webroot_basepath,
	$cgi_basepath,
	'/opt/monitor/op5/reports/gui',
	'/opt/monitor/op5/gui',
	'/opt/monitor/op5/backup',
	'/opt/monitor/op5/nacoma'
);


if( $cgi_cfg_unlink ) {
	register_shutdown_function(function() use ($cgi_cfg_path) {
		unlink( $cgi_cfg_path );
	});
}

/* ***************************************
 * Load path
 * ***************************************/

if( isset( $_SERVER['PATH_INFO'] ) ) {
	$path_info = $_SERVER['PATH_INFO'];
} else {
	$url = (isset($_SERVER['HTTPS'])&&$_SERVER['HTTPS']=='on')?'https://':'http://';
	$url .= $_SERVER['HTTP_HOST'];
	$url .= $_SERVER['SCRIPT_NAME'];
	$url .= '/';
	
	header( 'Location: '.$url );
	return;
}

$is_cgi = false;
if( substr( $path_info, 0, strlen($cgi_bin_urlbase) ) == $cgi_bin_urlbase ) {
	$filepath = $cgi_basepath . substr( $path_info, strlen($cgi_bin_urlbase) );
	$is_cgi = true;
} else  {
	$filepath = $webroot_basepath . $path_info;
}

$realpath = realpath($filepath);
if( is_dir( $realpath ) && is_file( $realpath."/index.php" ) ) {
	$realpath .= "/index.php";
}

/* ***************************************
 * Test so we only access allowed directories
 * ***************************************/
$allowed = false;
foreach( $allowed_dirs as $dir ) {
	if( substr( $realpath, 0, strlen($dir) ) == $dir ) {
		$allowed = true;
		break;
	}
}

if( empty($realpath) ) {
	/* Handle 404... */
	header("HTTP/1.0 404 Not Found");
	print "404 file not found\n";
	return;
}
else if( !$allowed ) {
	/* Handle 404... */
	header("HTTP/1.0 403 Forbidden");
	print "403 Forbidden\n";
	return;
}
/* ***************************************
 * Do Authentication and authorization
 * ***************************************/

$op5auth = Op5Auth::factory( array( 'session_key' => false ) );
try {
	if( !isset($_SERVER['PHP_AUTH_USER']) ||
	    !isset($_SERVER['PHP_AUTH_PW']) ||
	    !$op5auth->login( $_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'] ) )
	{
		header( 'WWW-Authenticate: Basic realm="op5 Monitor Old GUI"' );
		header( "HTTP/1.0 401 Unauthorized" );
		echo "401 Unauthorized, login to access montior.old gui";
		return;
	}
}
catch( Exception $e ) {
		header( 'WWW-Authenticate: Basic realm="op5 Monitor Old GUI"' );
		header( "HTTP/1.0 401 Unauthorized" );
		echo "401 Unauthorized, login to access montior.old gui";
		return;
}

$op5user = $op5auth->get_user();

$_SERVER['REMOTE_USER'] = $op5user->username;

/* ***************************************
 * Save authorization
 * ***************************************/

function build_cgi_cfg() {
	global $authz;
	global $cgi_cfg_template_path;
	global $cgi_cfg_path;
	global $op5user;

	$authz = array(
		'system_information'        => $op5user->authorized_for('system_information'),
		'configuration_information' => $op5user->authorized_for('configuration_information'),
		'system_commands'           => $op5user->authorized_for('system_commands'),
		'all_services'              => $op5user->authorized_for('service_view_all'),
		'all_hosts'                 => $op5user->authorized_for('host_view_all'),
		'all_service_commands'      => $op5user->authorized_for('service_edit_all'),
		'all_host_commands'         => $op5user->authorized_for('host_edit_all')
	);


	$cgi_cfg = file( $cgi_cfg_template_path );
	$cgi_cfg = array_map( function($row) {
		global $op5user;
		global $authz;
		global $url_html_path;
		if( preg_match( '/^url_html_path=(.*)$/', $row ) )
			return 'url_html_path='.$url_html_path;
		if( !preg_match( '/^authorized_for_([^=]+)=(.*)$/', $row, $matches ) )
			return trim($row,"\n\r");
		$users = array_filter(array_map( 'trim', explode(',',$matches[2]) ));

		/* Remove us first */
		$users = array_diff( $users, array($op5user->username) );

		if (isset($authz[$matches[1]]) && $authz[$matches[1]]) {
			$users[] = $op5user->username;
		}

		return 'authorized_for_'.$matches[1].'='.implode(',',$users);
	}, $cgi_cfg);

	$cgi_cfg_header = <<<EOF
###############################################################################
#
# THIS FILE IS AUTO GENERATED FROM THE FILE:
# $cgi_cfg_template_path
#
# Changes in this file will be overwritten, change above file instead
#
###############################################################################


EOF;

	file_put_contents( $cgi_cfg_path, $cgi_cfg_header.implode("\n",$cgi_cfg)."\n", LOCK_EX );
}

/* ***************************************
 * Initialize environment
 * ***************************************/
$_SERVER['DOCUMENT_ROOT'] = '/var/www/html';
$_SERVER['SCRIPT_FILENAME'] = $realpath;
/* No support for PATH_INFO passthru */
$_SERVER['SCRIPT_NAME'] = $_SERVER['REQUEST_URI'];
$_SERVER['PATH_INFO'] = '';

/* ***************************************
 * Load scripts
 * ***************************************/

if( substr($realpath, 0, strlen($cgi_basepath)) == $cgi_basepath && $is_cgi ) {
	/* Handle CGI */
	
	$_SERVER['DOCUMENT_ROOT'] = $cgi_basepath;

	foreach( $_SERVER as $key => $val ) {
		putenv( "$key=$val" );
	}
	putenv( "NAGIOS_CGI_CONFIG=".$cgi_cfg_path );
	
	/* Build post data */
	$post_data = file_get_contents('php://input');

	$descriptorspec = array(
		0 => array('pipe', 'r'),
		1 => array('pipe', 'w'),
		2 => array('file', $cgi_error_log, 'a')
		);

	build_cgi_cfg();
	
	$process = proc_open( $realpath, $descriptorspec, $pipes, dirname($realpath) );
	
	if( $process === false ) {
		header( "HTTP/1.0 500 Internal Server Error" );
		echo "500 Internal Server Error";
	}
	else {
		/* Send post data */
		fwrite($pipes[0], $post_data);
		fclose($pipes[0]);

		do {
			$line = trim( fgets($pipes[1]) );
			if( !empty($line) ) {
				header( $line );
			}
		} while( !empty($line) );
		fpassthru($pipes[1]);
		fclose($pipes[1]);

		$return_value = proc_close($process);
	}
}
else if( !empty($realpath) ) {
	if( substr( $realpath, -4 ) == '.php' ) {
		/* Handle PHP files */
		
		set_include_path( get_include_path() . PATH_SEPARATOR . dirname($realpath) );
		require( $realpath );
	} else {
		/* Handle Static files */

		/* There are php libraries to determine mimetype, but they ssem not to handle css */
		$mime = 'text/html';
		switch( strrchr( $realpath, '.' ) ) {
			case '.png':  $mime = 'image/png'; break;
			case '.gif':  $mime = 'image/gif'; break;
			case '.css':  $mime = 'text/css'; break;
			case '.js':   $mime = 'application/javascript'; break;
			case '.html': $mime = 'text/html'; break;
		}
		header('Content-Type: '.$mime);
		readfile( $realpath );
	}
}

exit(0);
